ALTER SESSION SET CURRENT_SCHEMA = XYZG_ADMIN;

CREATE TABLE TRAINING (
  ID NUMBER PRIMARY KEY,
  DESCRIPTION VARCHAR2(255)
);

CREATE TABLE STAFF (
  ID NUMBER PRIMARY KEY,
  FIRSTNAME VARCHAR2(32),
  LASTNAME VARCHAR2(32)
);

CREATE TABLE TRAINING_STAFF_LINK (
  TRAINING_ID NUMBER,
  STAFF_ID NUMBER,
  ATTENDANCE_DATE TIMESTAMP,
  CONSTRAINT FK_TSL_TRAINING FOREIGN KEY (TRAINING_ID) REFERENCES TRAINING(ID),
  CONSTRAINT FK_TSL_STAFF FOREIGN KEY (STAFF_ID) REFERENCES STAFF(ID)
);

CREATE TABLE TASK_TYPE (
  TYPE VARCHAR2(32) PRIMARY KEY,
  HOURLY_RATE NUMERIC(9,2)
);

CREATE TABLE JOB (
  ID NUMBER PRIMARY KEY,
  DESCRIPTION VARCHAR2(64) NOT NULL
);

CREATE TABLE TASK (
  ID NUMBER PRIMARY KEY,
  TIME_TO_COMPLETE NUMBER,
  JOB_ID NUMBER,
  TYPE VARCHAR2(32),
  CONSTRAINT FK_TASK_JOB_ID FOREIGN KEY (JOB_ID) REFERENCES JOB(ID),
  CONSTRAINT FK_TASK_TASK_TYPE FOREIGN KEY (TYPE) REFERENCES TASK_TYPE(TYPE)
);

CREATE TABLE CLIENT (
  ID NUMBER PRIMARY KEY,
  FIRSTNAME VARCHAR2(32),
  LASTNAME VARCHAR2(32)
);

CREATE TABLE SPECIAL_CONTRACT (
  ID NUMBER PRIMARY KEY,
  MONTHLY_FEE NUMERIC(9,2) NOT NULL,
  VALID_TO TIMESTAMP NOT NULL,
  CLIENT_ID NUMBER NOT NULL,
  CONSTRAINT FK_SC_CLIENT_ID FOREIGN KEY (CLIENT_ID) REFERENCES CLIENT(ID)
);

CREATE TABLE GARDEN (
  ID NUMBER PRIMARY KEY,
  ADDRESS VARCHAR2(32),
  CLIENT_ID NUMBER,
  CONSTRAINT FK_GARDEN_CLIENT_ID FOREIGN KEY (CLIENT_ID) REFERENCES CLIENT(ID)
);

CREATE TABLE GARDEN_JOB_LINK (
  GARDEN_ID NUMBER,
  JOB_ID NUMBER,
  CONSTRAINT FK_GJL_GARDEN FOREIGN KEY (GARDEN_ID) REFERENCES GARDEN(ID),
  CONSTRAINT FK_GJL_JOB FOREIGN KEY (JOB_ID) REFERENCES JOB(ID)
);

CREATE TABLE JOB_STAFF_LINK (
  JOB_ID NUMBER,
  STAFF_ID NUMBER,
  CONSTRAINT FK_JSL_JOB FOREIGN KEY (JOB_ID) REFERENCES JOB(ID),
  CONSTRAINT FK_JSL_STAFF FOREIGN KEY (STAFF_ID) REFERENCES STAFF(ID)
);

