ALTER SESSION SET CURRENT_SCHEMA = XYZG_ADMIN;

CREATE TABLE TRAINING (
  ID NUMBER PRIMARY KEY,
  DESCRIPTION VARCHAR2(255)
);

CREATE TABLE STAFF (
  ID NUMBER PRIMARY KEY,
  FIRSTNAME VARCHAR2(32),
  LASTNAME VARCHAR2(32)
);

CREATE TABLE TRAINING_STAFF_LINK (
  TRAINING_ID NUMBER,
  STAFF_ID NUMBER,
  ATTENDANCE_DATE TIMESTAMP,
  CONSTRAINT FK_TSL_TRAINING FOREIGN KEY (TRAINING_ID) REFERENCES TRAINING(ID),
  CONSTRAINT FK_TSL_STAFF FOREIGN KEY (STAFF_ID) REFERENCES STAFF(ID)
);

CREATE TABLE TASK_TYPE (
  TYPE VARCHAR2(32) PRIMARY KEY,
  HOURLY_RATE NUMERIC(9,2)
);

CREATE TABLE JOB (
  ID NUMBER PRIMARY KEY,
  DESCRIPTION VARCHAR2(64) NOT NULL
);

CREATE TABLE TASK (
  ID NUMBER PRIMARY KEY,
  TIME_TO_COMPLETE NUMBER,
  JOB_ID NUMBER,
  TYPE VARCHAR2(32),
  CONSTRAINT FK_TASK_JOB_ID FOREIGN KEY (JOB_ID) REFERENCES JOB(ID),
  CONSTRAINT FK_TASK_TASK_TYPE FOREIGN KEY (TYPE) REFERENCES TASK_TYPE(TYPE)
);

CREATE TABLE CLIENT (
  ID NUMBER PRIMARY KEY,
  FIRSTNAME VARCHAR2(32),
  LASTNAME VARCHAR2(32)
);

CREATE TABLE SPECIAL_CONTRACT (
  ID NUMBER PRIMARY KEY,
  MONTHLY_FEE NUMERIC(9,2) NOT NULL,
  VALID_TO TIMESTAMP NOT NULL,
  CLIENT_ID NUMBER NOT NULL,
  CONSTRAINT FK_SC_CLIENT_ID FOREIGN KEY (CLIENT_ID) REFERENCES CLIENT(ID)
);

CREATE TABLE GARDEN (
  ID NUMBER PRIMARY KEY,
  ADDRESS VARCHAR2(32),
  CLIENT_ID NUMBER,
  CONSTRAINT FK_GARDEN_CLIENT_ID FOREIGN KEY (CLIENT_ID) REFERENCES CLIENT(ID)
);

CREATE TABLE GARDEN_JOB_LINK (
  GARDEN_ID NUMBER,
  JOB_ID NUMBER,
  CONSTRAINT FK_GJL_GARDEN FOREIGN KEY (GARDEN_ID) REFERENCES GARDEN(ID),
  CONSTRAINT FK_GJL_JOB FOREIGN KEY (JOB_ID) REFERENCES JOB(ID)
);

CREATE TABLE JOB_STAFF_LINK (
  JOB_ID NUMBER,
  STAFF_ID NUMBER,
  CONSTRAINT FK_JSL_JOB FOREIGN KEY (JOB_ID) REFERENCES JOB(ID),
  CONSTRAINT FK_JSL_STAFF FOREIGN KEY (STAFF_ID) REFERENCES STAFF(ID)
);

--
-- task_type dml
--

INSERT INTO TASK_TYPE (TYPE, HOURLY_RATE) VALUES ('Assistance', 8);
INSERT INTO TASK_TYPE (TYPE, HOURLY_RATE) VALUES ('Cleaning', 8.99);
INSERT INTO TASK_TYPE (TYPE, HOURLY_RATE) VALUES ('Landscape works', 16);
INSERT INTO TASK_TYPE (TYPE, HOURLY_RATE) VALUES ('Consulting', 8.25);
INSERT INTO TASK_TYPE (TYPE, HOURLY_RATE) VALUES ('Other', 18.99);

--
-- staff dml
--

INSERT INTO STAFF (ID, FIRSTNAME, LASTNAME) VALUES (1, 'Anna', 'Lee');
INSERT INTO STAFF (ID, FIRSTNAME, LASTNAME) VALUES (2, 'Elizabeth', 'Doe');
INSERT INTO STAFF (ID, FIRSTNAME, LASTNAME) VALUES (3, 'John', 'Smith');
INSERT INTO STAFF (ID, FIRSTNAME, LASTNAME) VALUES (4, 'Eric', 'Johnson');
INSERT INTO STAFF (ID, FIRSTNAME, LASTNAME) VALUES (5, 'Sarah', 'Jenkins');

--
-- training dml
--

INSERT INTO TRAINING (ID, DESCRIPTION) VALUES (1, 'Do not throw blades!');
INSERT INTO TRAINING (ID, DESCRIPTION) VALUES (2, 'Chemicals are not for drinking');
INSERT INTO TRAINING (ID, DESCRIPTION) VALUES (3, 'Fire; Why do we worry?');
INSERT INTO TRAINING (ID, DESCRIPTION) VALUES (4, 'Be polite with clients');
INSERT INTO TRAINING (ID, DESCRIPTION) VALUES (5, 'Inventory using');

--
-- training_staff_link_dml
--

INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (1, 1, '01.12.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (2, 1, '12.08.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (3, 1, '10.05.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (4, 1, '28.04.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (2, 2, '04.12.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (1, 3, '11.11.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (4, 3, '03.06.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (3, 4, '15.12.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (2, 5, '16.09.2017');
INSERT INTO TRAINING_STAFF_LINK (TRAINING_ID, STAFF_ID, ATTENDANCE_DATE) VALUES (5, 5, '17.10.2017');

--
-- job dml
--

INSERT INTO JOB (ID, DESCRIPTION) VALUES (1, 'Prepare the area');
INSERT INTO JOB (ID, DESCRIPTION) VALUES (2, 'Cleaning the homestead');
INSERT INTO JOB (ID, DESCRIPTION) VALUES (3, 'Create the new garden project');
INSERT INTO JOB (ID, DESCRIPTION) VALUES (4, 'Maintaining the area');
INSERT INTO JOB (ID, DESCRIPTION) VALUES (5, 'Remove the old plants');
INSERT INTO JOB (ID, DESCRIPTION) VALUES (6, 'Prevent plant viruses');

--
-- task dml
--

INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (1, 20, 4, 'Assistance');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (2, 36, 1, 'Landscape works');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (3, 4, 5, 'Cleaning');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (4, 12, 6, 'Other');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (5, 6, 6, 'Consulting');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (6, 8, 3, 'Assistance');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (7, 11, 3, 'Landscape works');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (8, 72, 2, 'Other');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (9, 25, 4, 'Consulting');
INSERT INTO TASK (ID, TIME_TO_COMPLETE, JOB_ID, TYPE) VALUES (10, 6, 2, 'Cleaning');

--
-- job_staff_link dml
--

INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (1, 4);
INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (6, 5);
INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (3, 2);
INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (1, 1);
INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (4, 3);
INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (3, 1);
INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (2, 5);
INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (2, 3);
INSERT INTO JOB_STAFF_LINK (JOB_ID, STAFF_ID) VALUES (1, 2);

--
-- client dml
--

INSERT INTO CLIENT (ID, FIRSTNAME, LASTNAME) VALUES (1, 'Andy', 'Doe');
INSERT INTO CLIENT (ID, FIRSTNAME, LASTNAME) VALUES (2, 'Fred', 'Sunny');
INSERT INTO CLIENT (ID, FIRSTNAME, LASTNAME) VALUES (3, 'Monica', 'Sandman');
INSERT INTO CLIENT (ID, FIRSTNAME, LASTNAME) VALUES (4, 'Cristie', 'Tailor');
INSERT INTO CLIENT (ID, FIRSTNAME, LASTNAME) VALUES (5, 'Tom', 'Hawk');

--
-- garden dml
--

INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (1, 'Main st. 11', 4);
INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (2, 'Victory st. 66', 2);
INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (3, 'Total square 15', 1);
INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (4, 'Time square', 4);
INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (5, 'New Springfield 61', 5);
INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (6, 'San Fiero 12st street', 3);
INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (7, 'Green square 16', 5);
INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (8, 'Blessing st. 113', 3);
INSERT INTO GARDEN (ID, ADDRESS, CLIENT_ID) VALUES (9, 'Manny farm', 1);

--
-- garden_job_link dml
--

INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (5, 4);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (1, 2);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (1, 6);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (3, 1);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (4, 5);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (1, 1);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (2, 5);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (5, 2);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (2, 1);
INSERT INTO GARDEN_JOB_LINK (GARDEN_ID, JOB_ID) VALUES (3, 3);

--
-- special_contract dml
--

INSERT INTO SPECIAL_CONTRACT (ID, MONTHLY_FEE, VALID_TO, CLIENT_ID) VALUES (1, 144, '12.03.2018', 1);
INSERT INTO SPECIAL_CONTRACT (ID, MONTHLY_FEE, VALID_TO, CLIENT_ID) VALUES (2, 199, '27.02.2018', 4);
INSERT INTO SPECIAL_CONTRACT (ID, MONTHLY_FEE, VALID_TO, CLIENT_ID) VALUES (3, 99, '30.12.2017', 2);
INSERT INTO SPECIAL_CONTRACT (ID, MONTHLY_FEE, VALID_TO, CLIENT_ID) VALUES (4, 275, '10.07.2018', 3);

--
-- 1
--

SELECT *
FROM CLIENT;

--
-- 2
--

SELECT
  g.ID,
  g.ADDRESS,
  concat(c.LASTNAME, concat(' ', c.FIRSTNAME))
FROM GARDEN g
  JOIN CLIENT c ON g.CLIENT_ID = c.ID
WHERE CLIENT_ID IS NOT NULL
ORDER BY c.LASTNAME;

--
-- 3
--

SELECT
  concat(c.FIRSTNAME, concat(' ', c.LASTNAME)) AS client_fullname,
  g.ADDRESS,
  sc.MONTHLY_FEE
FROM GARDEN g
  JOIN CLIENT c ON g.CLIENT_ID = c.ID
  JOIN SPECIAL_CONTRACT sc ON sc.CLIENT_ID = c.ID;

--
-- 4
--

SELECT
  t.type,
  j.DESCRIPTION
FROM (SELECT *
      FROM (SELECT
              tt.TYPE,
              count(*) AS counter
            FROM TASK_TYPE tt
              JOIN TASK t ON tt.TYPE = t.TYPE
              JOIN JOB j ON t.JOB_ID = j.ID
            GROUP BY tt.TYPE
            ORDER BY counter DESC)
      WHERE ROWNUM = 1) mf_type
  JOIN TASK t ON t.TYPE = mf_type.TYPE
  JOIN JOB j ON t.JOB_ID = j.ID;

--
-- 5
--

WITH exp AS (
SELECT
STAFF_ID,
count(*) trainings_count
FROM TRAINING_STAFF_LINK
GROUP BY STAFF_ID
),

most_experienced_staff AS (
SELECT
ID,
FIRSTNAME,
LASTNAME,
trainings_count
FROM exp
JOIN STAFF s ON exp.STAFF_ID = s.ID
WHERE ROWNUM = 1
)

SELECT
  j.DESCRIPTION,
  mes.LASTNAME,
  mes.trainings_count
FROM most_experienced_staff mes
  JOIN JOB_STAFF_LINK jsl ON mes.ID = jsl.STAFF_ID
  JOIN JOB j ON jsl.JOB_ID = j.ID
WHERE ROWNUM = 1;

--
-- 6
--

WITH clients_without_contract AS (
SELECT c.*
FROM CLIENT c LEFT JOIN SPECIAL_CONTRACT sc ON c.ID = sc.CLIENT_ID
WHERE sc.ID IS NULL
)

SELECT
  cwc.ID,
  concat(cwc.FIRSTNAME, concat(' ', cwc.LASTNAME)) AS client_full_name,
  sum(t.TIME_TO_COMPLETE * tt.HOURLY_RATE) AS total_spent
FROM GARDEN g
  JOIN clients_without_contract cwc ON g.CLIENT_ID = cwc.ID
  JOIN GARDEN_JOB_LINK gjl ON gjl.GARDEN_ID = g.ID
  JOIN JOB j ON gjl.JOB_ID = j.ID
  JOIN TASK t ON j.ID = t.JOB_ID
  JOIN TASK_TYPE tt ON t.TYPE = tt.TYPE GROUP BY cwc.ID, cwc.LASTNAME, cwc.FIRSTNAME;

DROP TABLE TRAINING_STAFF_LINK;
DROP TABLE TRAINING;
DROP TABLE JOB_STAFF_LINK;
DROP TABLE STAFF;
DROP TABLE TASK;
DROP TABLE TASK_TYPE;
DROP TABLE GARDEN_JOB_LINK;
DROP TABLE JOB;
DROP TABLE SPECIAL_CONTRACT;
DROP TABLE GARDEN;
DROP TABLE CLIENT;